import ts from 'typescript';
import path from 'path';
import fs from 'fs';
import nodeResolve from '@rollup/plugin-node-resolve';

var tsPathsResolve = function (_a) {
    var _b = _a === void 0 ? {} : _a, _c = _b.tsConfigPath, tsConfigPath = _c === void 0 ? process.env["TS_NODE_PROJECT"] || ts.findConfigFile(".", ts.sys.fileExists) || "tsconfig.json" : _c, _d = _b.logLevel, logLevel = _d === void 0 ? "warn" : _d, _e = _b.fallback, fallback = _e === void 0 ? nodeResolve() : _e;
    var pluginName = "ts-paths";
    var compilerOptions = getTsConfig(tsConfigPath, logLevel, pluginName);
    var mappings = createMappings(compilerOptions, logLevel, pluginName);
    return {
        name: pluginName,
        resolveId: function (request, importer) {
            if (typeof importer === "undefined" || request.startsWith("\0")) {
                return null;
            }
            var _a = findResolve({
                compilerOptions: compilerOptions,
                mappings: mappings,
                request: request,
                importer: importer,
            }), resolved = _a[0], nodeModules = _a[1];
            if (resolved) {
                if (logLevel === "debug") {
                    console.log(formatLog("info", pluginName, request + " -> " + resolved));
                }
                if (nodeModules) {
                    return fallback.resolveId(resolved, importer);
                }
                return resolved;
            }
            return fallback.resolveId(request, importer);
        },
    };
};
function formatLog(level, name, value) {
    switch (level) {
        case "error":
            return "\u001B[1;31m(!) [" + name + "]: " + value + "\u001B[0m";
        case "warn":
            return "\u001B[1;33m(!) [" + name + "]: " + value + "\u001B[0m";
        default:
            return "\u001B[1;34m(!) [" + name + "]: " + value + "\u001B[0m";
    }
}
var getTsConfig = function (tsConfigPath, logLevel, pluginName) {
    var _a = ts.readConfigFile(tsConfigPath, ts.sys.readFile), error = _a.error, config = _a.config;
    if (error) {
        throw new Error(formatLog("error", pluginName, error.messageText));
    }
    var _b = ts.parseJsonConfigFileContent(config, ts.sys, path.dirname(tsConfigPath)), errors = _b.errors, compilerOptions = _b.options;
    if (errors.length > 0) {
        throw new Error(formatLog("error", pluginName, errors.map(function (err) { return err.messageText.toString(); }).join("\n")));
    }
    if (!compilerOptions) {
        throw new Error(formatLog("error", pluginName, "'compilerOptions' is gone."));
    }
    if (!compilerOptions.baseUrl) {
        throw new Error(formatLog("error", pluginName, "Option 'compilerOptions.paths' cannot be used without specifying 'compilerOptions.baseUrl' option."));
    }
    if (!compilerOptions.paths || Object.keys(compilerOptions.paths).length === 0) {
        compilerOptions.paths = {};
        logLevel != "none" && console.warn(formatLog("warn", pluginName, "typescript compilerOptions.paths are empty."));
    }
    return compilerOptions;
};
var createMappings = function (compilerOptions, logLevel, pluginName) {
    var countWildcard = function (value) { var _a; return (_a = value.match(/\*/g)) === null || _a === void 0 ? void 0 : _a.length; };
    var valid = function (value) { return /(\*|\/\*|\/\*\/)/.test(value); };
    var mappings = [];
    for (var _i = 0, _a = Object.keys(compilerOptions.paths); _i < _a.length; _i++) {
        var pattern = _a[_i];
        if (countWildcard(pattern) > 1) {
            logLevel != "none" &&
                console.warn(formatLog("warn", pluginName, "path pattern '" + pattern + "' can have at most one '*' character."));
            continue;
        }
        var wildcard = pattern.indexOf("*");
        if (wildcard !== -1 && !valid(pattern)) {
            logLevel != "none" && console.warn(formatLog("warn", pluginName, "path pattern '" + pattern + "' is not valid."));
            continue;
        }
        var targets = compilerOptions.paths[pattern].filter(function (target) {
            var wildcard = target.indexOf("*");
            if (wildcard !== -1 && !valid(target)) {
                logLevel != "none" &&
                    console.warn(formatLog("warn", pluginName, "target pattern '" + target + "' is not valid"));
                return false;
            }
            if (target.indexOf("@types") !== -1 || target.endsWith(".d.ts")) {
                logLevel != "none" && console.warn(formatLog("warn", pluginName, "type defined " + target + " is ignored."));
                return false;
            }
            return true;
        });
        if (targets.length == 0) {
            continue;
        }
        if (pattern === "*") {
            mappings.push({ alias: { wildcard: true, pattern: pattern, prefix: "", suffix: "" }, targets: targets });
            continue;
        }
        mappings.push({
            alias: {
                wildcard: wildcard !== -1,
                pattern: pattern,
                prefix: pattern.substr(0, wildcard),
                suffix: pattern.substr(wildcard + 1),
            },
            targets: targets,
        });
    }
    if (logLevel === "debug") {
        for (var _b = 0, mappings_1 = mappings; _b < mappings_1.length; _b++) {
            var mapping = mappings_1[_b];
            console.log(formatLog("info", pluginName, "pattern: '" + mapping.alias.pattern + "' targets: '" + mapping.targets + "'"));
        }
    }
    return mappings;
};
var findResolve = function (_a) {
    var mappings = _a.mappings, request = _a.request, importer = _a.importer, compilerOptions = _a.compilerOptions;
    var longestMatchedPrefixLength = 0;
    var matched = undefined;
    for (var _i = 0, mappings_2 = mappings; _i < mappings_2.length; _i++) {
        var mapping = mappings_2[_i];
        var _b = mapping.alias, wildcard = _b.wildcard, prefix = _b.prefix, suffix = _b.suffix, source = _b.pattern;
        if (wildcard &&
            request.length >= prefix.length + suffix.length &&
            request.startsWith(prefix) &&
            request.endsWith(suffix)) {
            if (longestMatchedPrefixLength < prefix.length) {
                longestMatchedPrefixLength = prefix.length;
                matched = mapping;
            }
        }
        else if (request === source) {
            matched = mapping;
            break;
        }
    }
    if (!matched) {
        return ["", false];
    }
    var matchedWildcard = request.substr(matched.alias.prefix.length, request.length - matched.alias.suffix.length);
    for (var _c = 0, _d = matched.targets; _c < _d.length; _c++) {
        var target = _d[_c];
        var predicted = target;
        if (matched.alias.wildcard) {
            predicted = target.replace("*", matchedWildcard);
        }
        var answer = path.resolve(compilerOptions.baseUrl, predicted);
        if (answer.indexOf("node_modules/") != -1) {
            return [answer, true];
        }
        var result = ts.resolveModuleName(answer, importer, compilerOptions, ts.sys);
        if (result === null || result === void 0 ? void 0 : result.resolvedModule) {
            return [result.resolvedModule.resolvedFileName, false];
        }
        if (fs.existsSync(answer)) {
            return [answer, false];
        }
    }
    return ["", false];
};

export default tsPathsResolve;
export { tsPathsResolve };
//# sourceMappingURL=index.esm.js.map
